# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

import enum
from typing import List

from stytch.consumer.models.fraud import VerdictReasonAction
from stytch.core.response_base import ResponseBase


class OverrideRequestAction(str, enum.Enum):
    ALLOW = "ALLOW"
    CHALLENGE = "CHALLENGE"
    BLOCK = "BLOCK"
    NONE = "NONE"


class ListResponse(ResponseBase):
    """Response type for `VerdictReasons.list`.
    Fields:
      - verdict_reason_actions: Information about verdict reasons and any overrides that were set on them.
    """  # noqa

    verdict_reason_actions: List[VerdictReasonAction]


class OverrideResponse(ResponseBase):
    """Response type for `VerdictReasons.override`.
    Fields:
      - verdict_reason_action: Information about the verdict reason override that was just set.
    """  # noqa

    verdict_reason_action: VerdictReasonAction
